@@ -2,13 +2,15 @@
 ### Encoding: UTF-8
 
 ###################################################
-### code chunk number 1: simstudy_survival.Rnw:13-36
+### code chunk number 1: simstudy_survival.Rnw:13-42
 ###################################################
+# Chunk 1
 library(CALIBERrfimpute)
 library(missForest)
 library(survival)
 library(xtable)
 library(rpart)
+library(mice)
 
 kPmiss <- 0.2 # probability of missingness
 kLogHR <- 0.5 # true log hazard ratio
@@ -28,10 +30,15 @@ if (!exists('NPATS')){
 # e.g.
 # N <- 1000
 
+# To use more than 4 imputations, set NIMPS to a number greater than 4
+# e.g.
+# NIMPS <- 10
+
 
 ###################################################
-### code chunk number 2: simstudy_survival.Rnw:79-308
+### code chunk number 2: simstudy_survival.Rnw:85-339
 ###################################################
+# Chunk 2
 
 #### DATA GENERATING FUNCTIONS ####
 
@@ -54,7 +61,8 @@ makeSurv <- function(n = 2000, loghr = kLogHR){
 	obstime <- runif(nrow(data), min = 0,
 		max = quantile(data$survtime, 0.5))
 	data$event <- as.integer(data$survtime <= obstime)
-	data$time <- pmin(data$survtime, obstime)
+	# Generate integer survival times
+	data$time <- ceiling(100 * pmin(data$survtime, obstime))
 	
 	# Observed marginal cumulative hazard for imputation models
 	data$cumhaz <- nelsonaalen(data, time, event) 
@@ -85,6 +93,11 @@ makeMarSurv <- function(data, pmissing = kPmiss){
 		}
 		stepsize <- 32
 		lptrial <- lp
+		# To avoid errors due to missing linear predictors (ideally
+		# there should not be any missing), replace with the mean
+		if (any(is.na(lptrial))){
+			lp[is.na(lptrial)] <- mean(lptrial, na.rm = TRUE)
+		}
 		while(abs(trialn(lptrial) - n) > 1){
 			if (trialn(lptrial) > n){
 				# trialn bigger than required 
@@ -203,26 +216,44 @@ mice.impute.rfcont100 <- function(y, ry, x, ...){
 
 coxfull <- function(data){
 	# Full data analysis
-	coefs <- summary(coxph(myformula, data = data))$coef
-	# return a vector of coefficients (est), upper and lower 95% limits
-	confint <- cbind(coefs[, 'coef'] - qnorm(0.975) * coefs[, 'se(coef)'],
-		coefs[, 'coef'] + qnorm(0.975) * coefs[, 'se(coef)'])
-	out <- cbind(coefs[, 'coef'], confint,
-		kLogHR >= confint[,1] & kLogHR <= confint[,2])
-	colnames(out) <- c('est', 'lo 95', 'hi 95', 'cover')
+	coefs <- as.data.frame(summary(coxph(myformula, data = data))$coef)
+	# return a data.frame of coefficients (est), upper and lower 95% limits
+	out <- data.frame(est = coefs[, 'coef'], 
+		lo95 = coefs[, 'coef'] + qnorm(0.025) * coefs[, 'se(coef)'],
+		hi95 = coefs[, 'coef'] + qnorm(0.975) * coefs[, 'se(coef)'],
+		row.names = row.names(coefs))
+	out$cover <- kLogHR >= out$lo95 & kLogHR <= out$hi95
 	out
 }
 
 coximpute <- function(imputed_datasets){
 	# Analyses a list of imputed datasets
 	docoxmodel <- function(data){
-		coxph(myformula, data=data)
+		coxph(myformula, data = data)
 	}
 	mirafits <- as.mira(lapply(imputed_datasets, docoxmodel))
-	out <- summary(pool(mirafits))
-	out <- cbind(out, kLogHR >= out[, 'lo 95'] & kLogHR <= out[, 'hi 95'])
+	coefs <- as.data.frame(summary(pool(mirafits)))
+	if ('term' %in% colnames(coefs)){
+		row.names(coefs) <- as.character(coefs$term)
+	}
+	if (!('lo 95' %in% colnames(coefs))){
+		# newer version of mice
+		# use normal approximation for now, as assume large sample
+		# and large degrees of freedom for t distribution
+		out <- data.frame(est = coefs$estimate,
+			lo95 = coefs$estimate + qnorm(0.025) * coefs$std.error,
+			hi95 = coefs$estimate + qnorm(0.975) * coefs$std.error,
+			row.names = row.names(coefs))
+	} else if ('lo 95' %in% colnames(coefs)){
+		# older version of mice
+		out <- data.frame(est = coefs$est,
+			lo95 = coefs[, 'lo 95'], hi95 = coefs[, 'hi 95'],
+			row.names = row.names(coefs))
+	} else {
+		stop('Unable to handle format of summary.mipo object')
+	}
 	# Whether this confidence interval contains the true hazard ratio
-	colnames(out)[length(colnames(out))] <- 'cover'
+	out$cover <- kLogHR >= out$lo95 & kLogHR <= out$hi95
 	out
 }
 
@@ -264,8 +295,10 @@ doanalysis <- function(x){
 
 
 ###################################################
-### code chunk number 3: simstudy_survival.Rnw:312-316
+### code chunk number 3: simstudy_survival.Rnw:343-349
 ###################################################
+# Chunk 3
+
 mydata <- makeSurv(200)
 plot(mydata[, c('x1', 'x2', 'x3')],
 	main = "Associations between predictor variables in a sample dataset")
@@ -273,14 +306,18 @@ mydata <- makeSurv(20000)
 
 
 ###################################################
-### code chunk number 4: simstudy_survival.Rnw:321-322
+### code chunk number 4: simstudy_survival.Rnw:354-357
 ###################################################
+# Chunk 4
+
 summary(lm(x3 ~ x1*x2, data = mydata))
 
 
 ###################################################
-### code chunk number 5: simstudy_survival.Rnw:325-336
+### code chunk number 5: simstudy_survival.Rnw:360-373
 ###################################################
+# Chunk 5
+
 mydata <- makeSurv(2000)
 mydata2 <- makeMarSurv(mydata)
 # Plot non-missing data
@@ -295,8 +332,10 @@ title('Association of predictor variables x1 and x3')
 
 
 ###################################################
-### code chunk number 6: simstudy_survival.Rnw:341-365
+### code chunk number 6: simstudy_survival.Rnw:378-404
 ###################################################
+# Chunk 6
+
 # Cox proportional hazards analysis
 myformula <- as.formula(Surv(time, event) ~ x1 + x2 + x3)
 
@@ -324,8 +363,10 @@ summary(coxph(myformula, data = simdata))
 
 
 ###################################################
-### code chunk number 7: simstudy_survival.Rnw:387-405
+### code chunk number 7: simstudy_survival.Rnw:426-446
 ###################################################
+# Chunk 7
+
 # Setting analysis parameters: To analyse more than 3 samples,
 # set N to the desired number before running this program
 if (!exists('N')){
@@ -334,7 +375,7 @@ if (!exists('N')){
 # Number of imputations (set to at least 10 when
 # running an actual simulation)
 if (!exists('NIMPS')){
-	NIMPS <- 3
+	NIMPS <- 4
 }
 # Use parallel processing if the 'parallel' package is loaded
 if ('parallel' %in% loadedNamespaces() &&
@@ -347,8 +388,10 @@ if ('parallel' %in% loadedNamespaces() &&
 
 
 ###################################################
-### code chunk number 8: simstudy_survival.Rnw:434-470
+### code chunk number 8: simstudy_survival.Rnw:475-514
 ###################################################
+# Chunk 8
+
 getParams <- function(coef, method){
 	estimates <- sapply(results, function(x){
 		x[[method]][coef, 'est']
@@ -357,7 +400,7 @@ getParams <- function(coef, method){
 	se_bias <- sd(estimates) / sqrt(length(estimates))
 	mse <- mean((estimates - kLogHR) ^ 2)
 	ci_len <- mean(sapply(results, function(x){
-		x[[method]][coef, 'hi 95'] - x[[method]][coef, 'lo 95']
+		x[[method]][coef, 'hi95'] - x[[method]][coef, 'lo95']
 	}))
 	ci_cov <- mean(sapply(results, function(x){
 		x[[method]][coef, 'cover']
@@ -382,32 +425,41 @@ showTable <- function(coef){
 		'95% CI'), c('Bias', 'error of bias', 'square error', 'estimate',
 		'CI length', 'coverage'), out)
 	out <- cbind(c('', '', methodnames), out)
+	rownames(out) <- NULL
 	print(xtable(out), floating = FALSE, include.rownames = FALSE,
 		include.colnames = FALSE, hline.after = c(0, 2, nrow(out)))
 }
 
 
 ###################################################
-### code chunk number 9: simstudy_survival.Rnw:483-484
+### code chunk number 9: simstudy_survival.Rnw:527-530
 ###################################################
+# Chunk 9
+
 showTable('x1')
 
 
 ###################################################
-### code chunk number 10: simstudy_survival.Rnw:493-494
+### code chunk number 10: simstudy_survival.Rnw:539-542
 ###################################################
+# Chunk 10
+
 showTable('x2')
 
 
 ###################################################
-### code chunk number 11: simstudy_survival.Rnw:504-505
+### code chunk number 11: simstudy_survival.Rnw:552-555
 ###################################################
+# Chunk 11
+
 showTable('x3')
 
 
 ###################################################
-### code chunk number 12: simstudy_survival.Rnw:515-537
+### code chunk number 12: simstudy_survival.Rnw:565-589
 ###################################################
+# Chunk 12
+
 numtrees <- c(5, 10, 20, 100)
 bias <- sapply(numtrees, function(x){
 	getParams('x3', paste('rf', x, sep=''))['bias']
@@ -423,7 +475,7 @@ plot(-100, 0, type = 'p', pch = 15, cex = 1.3, ylab = 'Bias',
 	xlab = 'Number of trees', xlim = c(0,100),
 	ylim = c(min(lower_bias), max(upper_bias)))
 # Zero bias line
-lines(c(0,100), c(0,0), lty = 2, color = 'gray')
+lines(c(0,100), c(0,0), lty = 2, col = 'gray')
 # Confidence interval lines
 for (i in 1:5){lines(rep(numtrees[i], 2),
 	c(lower_bias[i], upper_bias[i]))}
@@ -433,8 +485,10 @@ title('Bias in estimate of x3 coefficient after\nmultiple imputation using RFcon
 
 
 ###################################################
-### code chunk number 13: simstudy_survival.Rnw:542-689
+### code chunk number 13: simstudy_survival.Rnw:594-745
 ###################################################
+# Chunk 13
+
 # Comparing confidence interval coverage and bias between:
 #    RF MICE 100 trees
 #    RF MICE 10 trees
@@ -510,12 +564,12 @@ compareCIlength <- function(method1, method2){
 	compareCIlengthVar <- function(varname){
 		# Paired t test for bias (difference in estimate)
 		len1 <- sapply(results, function(x){
-			x[[method1]][varname, 'hi 95'] -
-				x[[method1]][varname, 'lo 95']
+			x[[method1]][varname, 'hi95'] -
+				x[[method1]][varname, 'lo95']
 		})
 		len2 <- sapply(results, function(x){
-			x[[method2]][varname, 'hi 95'] -
-				x[[method2]][varname, 'lo 95']
+			x[[method2]][varname, 'hi95'] -
+				x[[method2]][varname, 'lo95']
 		})
 		
 		paste(formatC(mean(len1) / mean(len2),
@@ -562,6 +616,7 @@ maketable <- function(comparison){
 		c('Coefficient', 'parametric MICE',
 			'parametric MICE', 'RFcont MICE 100'),
 		compare)
+	rownames(compare) <- NULL
 	print(xtable(compare), include.rownames = FALSE,
 		include.colnames = FALSE, floating = FALSE,
 		hline.after = c(0, 2, nrow(compare)))
@@ -578,6 +633,7 @@ maketable <- function(comparison){
 		c('Coefficient', 'RFcont MICE 10',
 			'CART MICE', 'RF Doove MICE 100'),
 		compare)
+	rownames(compare) <- NULL
 	print(xtable(compare), include.rownames = FALSE,
 		include.colnames = FALSE, floating = FALSE,
 		hline.after = c(0, 2, nrow(compare)))
@@ -585,32 +641,42 @@ maketable <- function(comparison){
 
 
 ###################################################
-### code chunk number 14: simstudy_survival.Rnw:698-699
+### code chunk number 14: simstudy_survival.Rnw:754-757
 ###################################################
+# Chunk 14
+
 maketable(compareBias)
 
 
 ###################################################
-### code chunk number 15: simstudy_survival.Rnw:708-709
+### code chunk number 15: simstudy_survival.Rnw:766-769
 ###################################################
+# Chunk 15
+
 maketable(compareVariance)
 
 
 ###################################################
-### code chunk number 16: simstudy_survival.Rnw:719-720
+### code chunk number 16: simstudy_survival.Rnw:779-782
 ###################################################
+# Chunk 16
+
 maketable(compareCIlength)
 
 
 ###################################################
-### code chunk number 17: simstudy_survival.Rnw:729-730
+### code chunk number 17: simstudy_survival.Rnw:791-794
 ###################################################
+# Chunk 17
+
 maketable(compareCoverage)
 
 
 ###################################################
-### code chunk number 18: simstudy_survival.Rnw:768-777
+### code chunk number 18: simstudy_survival.Rnw:832-843
 ###################################################
+# Chunk 18
+
 showfunction <- function(functionname){
 	cat(paste(functionname, '<-',
 		paste(capture.output(print(get(functionname))),
@@ -623,8 +689,10 @@ showfunction('makeMarSurv')
 
 
 ###################################################
-### code chunk number 19: simstudy_survival.Rnw:782-794
+### code chunk number 19: simstudy_survival.Rnw:848-862
 ###################################################
+# Chunk 19
+
 showfunction('coxfull')
 showfunction('coximpute')
 showfunction('domissf')
@@ -640,8 +708,10 @@ showfunction('doanalysis')
 
 
 ###################################################
-### code chunk number 20: simstudy_survival.Rnw:799-804
+### code chunk number 20: simstudy_survival.Rnw:867-874
 ###################################################
+# Chunk 20
+
 showfunction('pstar')
 showfunction('compareBias')
 showfunction('compareVariance')
@@ -650,8 +720,10 @@ showfunction('compareCoverage')
 
 
 ###################################################
-### code chunk number 21: simstudy_survival.Rnw:809-812
+### code chunk number 21: simstudy_survival.Rnw:879-884
 ###################################################
+# Chunk 21
+
 showfunction('getParams')
 showfunction('showTable')
 showfunction('maketable')
